/***********************************************************************
*
* Copyright (c) 2021-2022 Barbara Geller and Ansel Sermersheim
*
* This example is free software, released under the BSD 2-Clause license.
* For license details refer to LICENSE provided with this project.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* https://opensource.org/licenses/BSD-2-Clause
*
***********************************************************************/

#include <QtCore>
#include <QtGui>

#include <QSqlTableModel>
#include <QSqlQuery>

class MainWindow : public QWidget
{
 public:
   MainWindow();

 private:
   QTableView *m_view;
   QSqlTableModel *m_sqlModel;
   QSqlDatabase m_db;

   bool createConnection();
};

MainWindow::MainWindow()
{
   setMinimumSize(800, 500);

   QLabel *label = new QLabel();
   label->setText("Table View");
   label->setAlignment(Qt::AlignHCenter);

   QFont font = label->font();
   font.setPointSize(11);
   label->setFont(font);

   if (! createConnection()) {
      std::exit(EXIT_FAILURE);
   }

   m_sqlModel = new QSqlTableModel(this, m_db);
   m_sqlModel->setTable("famousPeople");

   m_sqlModel->setSort(2, Qt::AscendingOrder);
   m_sqlModel->select();

   m_sqlModel->setHeaderData(1, Qt::Horizontal, QString("First Name"));
   m_sqlModel->setHeaderData(2, Qt::Horizontal, QString("Last Name"));
   m_sqlModel->setHeaderData(3, Qt::Horizontal, QString("Professional Biography"));

   //
   m_view = new QTableView();
   m_view->setModel(m_sqlModel);
   m_view->horizontalHeader()->setSectionsMovable(true);
   m_view->horizontalHeader()->setStretchLastSection(true);

   m_view->hideColumn(0);
   m_view->setColumnWidth(1, 100);
   m_view->setColumnWidth(2, 100);

   QPushButton *pb_close = new QPushButton();
   pb_close->setText("Close");

   QHBoxLayout *layout = new QHBoxLayout();
   layout->addStretch();
   layout->addWidget(pb_close);
   layout->addStretch();

   QVBoxLayout *layout_main = new QVBoxLayout(this);
   layout_main->setContentsMargins(20, 20, 20, 15);

   layout_main->addWidget(label);
   layout_main->addSpacing(10);
   layout_main->addWidget(m_view);
   layout_main->addSpacing(10);
   layout_main->addLayout(layout);

   connect(pb_close, &QPushButton::clicked, this, &QWidget::close);
}

bool MainWindow::createConnection()
{
   m_db = QSqlDatabase::addDatabase("QSQLITE", "peopleDB");
   m_db.setDatabaseName(":memory:");

   if (! m_db.open()) {
      QMessageBox::critical(nullptr, "Error Opening Database",
            "Unable to establish a connection to the SQLite database.",
             QMessageBox::Cancel);

      return false;
   }

   QSqlQuery query(m_db);
   query.exec("create table famousPeople (id int primary key, "
        "firstname varchar(20), lastname varchar(20), bio varchar(200))");

   query.exec("insert into famousPeople values(1, 'Gordon',  'Lightfoot', "
        "'Canadian folk singer, wrote ballads which told elaborate stories.')");

   query.exec("insert into famousPeople values(2, 'George', 'Carlin', "
        "'Comedian, author, social critic. Known for his opinions about politics, "
        "language, religion, and taboo subjects.')");

   query.exec("insert into famousPeople values(3, 'Stephen', 'Sondheim', "
         "'Composer and lyricist of musical theater. Credits include Sweeney Todd, "
         "Sunday in the Park with George, and Into the Woods.')");

   query.exec("insert into famousPeople values(4, 'Dennis', 'Ritchie', "
         "'A key developer of the C programming language.')");

   query.exec("insert into famousPeople values(5, 'Robert', 'Heinlein', "
         "'Published 32 science fiction novels.')");

   query.exec("insert into famousPeople values(6, 'Shirley', 'Corriher', "
         "'Biochemist, cookbook author, James Beard foundation award winner. If you want "
         "to understand the chemistry of baking her books are fantastic.')");

   query.exec("insert into famousPeople values(7, 'Vincent', 'Van Gogh', "
         "'Dutch painter with a very unique and special style, influenced modern art.')");

   return true;
}

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   MainWindow *window = new MainWindow();
   window->show();

   return app.exec();
}
